<?php

class curl_util
{
    function send_and_load ($url, parameter_integrator $parameters, $userpwd=null)
    {
        $posted_parameters = $parameters->marshall();
        $parameters_class = get_class($parameters);

        if (isset($_COOKIE['mode_developer']))
        {
            create_log("Step 1: send payment request",
                "", "",
                "URL", $url,
                "Method", "POST");
        }

        if (in_array($parameters_class, explode(",", PARAMETER_CLASSES_XML)))
        {
            $param_name = is_null($userpwd) ? "load" : "requestXML";
            create_log(null, "Parameters", "", $param_name, $posted_parameters);
            return self::send_xml($url, $posted_parameters, $param_name, $userpwd);
        }
        elseif (in_array($parameters_class, explode(",", PARAMETER_CLASSES_WPF)))
        {
            create_log(null, null, null, 
                "POST Parameters", $parameters->as_wpf(true));
            return self::send_wpf($url, $posted_parameters);
        }
        elseif (in_array($parameters_class, explode(",", PARAMETER_CLASSES_HPP)))
        {
            $secreet = $parameters->identification_unique;
            $posted_params = array(
                $parameters->payment_method->field_tx_secret_ => $secreet,
            );

            $encoded_posted_params = http_build_query($posted_params, '', '&');
            create_log(null, "REQUEST", null, 
                    "URL", "$url",
                    "transactionSecret", $secreet,
                    "requestXML", $posted_parameters);

            return self::send_xml($url, $posted_parameters, $encoded_posted_params .'&requestXML');
        }
        else
        {
            throw new Exception('invalid parameter_integrator Object');
        }
    }

    function send_xml ($url, $xml_string, $param_name = "load", $userpwd = null, $_http_method = CURLOPT_POSTFIELDS)
    {
        create_log(null, "XML Request URL", "", "URL", $url);

        $ch = curl_init();
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Content-Type: application/x-www-form-urlencoded; charset=UTF-8"
        ));

        curl_setopt($ch, CURLOPT_URL, $url);
        if (!empty ($xml_string))
        {
	    $encoded_xml_string = utf8_encode($xml_string);
            curl_setopt($ch, $_http_method, $param_name. '=' . urlencode($encoded_xml_string));
        }

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        //curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
        curl_setopt($ch, CURLOPT_USERAGENT, "php ctpepost");

        if (!is_null($userpwd))
        {
        	create_log('', '', '', "Authorization", $userpwd);
            curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            curl_setopt($ch, CURLOPT_USERPWD, $userpwd);
        }

        $result = curl_exec($ch);
        self::check_curl($ch);

        $info = curl_getinfo($ch);
        create_log('', '', '', "Request Info", print_r($info, true));

        curl_close($ch);
        return $result;
    }

    function send_wpf ($url, $wpf_string, $_is_logged = true)
    {
        $cpt = curl_init();

        curl_setopt($cpt, CURLOPT_HTTPHEADER, array(
            "Content-Type: application/x-www-form-urlencoded; charset=UTF-8"
        ));

        curl_setopt($cpt, CURLINFO_HEADER_OUT, true);
        curl_setopt($cpt, CURLOPT_URL, $url);
        curl_setopt($cpt, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($cpt, CURLOPT_USERAGENT, "php ctpepost");
        curl_setopt($cpt, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($cpt, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($cpt, CURLOPT_POST, 1);
        curl_setopt($cpt, CURLOPT_POSTFIELDS, $wpf_string);

        $result = curl_exec($cpt);
        self::check_curl($cpt);
        
        if (function_exists("create_log") && $_is_logged)
        {
            $info = curl_getinfo($cpt);
            create_log('', '', '', "Request Info", print_r($info, true));
        }

        curl_close($cpt);
        return $result;
    }

    function send_post ($url, $post_string, $_is_logged = true)
    {
        return self::send_wpf($url, $post_string, $_is_logged);
    }

    function send_get ($_url)
    {
        $cpt = curl_init();

        curl_setopt($cpt, CURLOPT_HTTPHEADER, array(
            "Content-Type: application/x-www-form-urlencoded; charset=UTF-8"
        ));

        curl_setopt($cpt, CURLINFO_HEADER_OUT, true);
        curl_setopt($cpt, CURLOPT_URL, $_url);
        curl_setopt($cpt, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($cpt, CURLOPT_USERAGENT, "php ctpepost");
        curl_setopt($cpt, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($cpt, CURLOPT_SSL_VERIFYPEER, FALSE);

        $result = curl_exec($cpt);
        self::check_curl($cpt);

        curl_close($cpt);
        return $result;
    }

    function check_curl ($_curl)
    {
        $curl_info = curl_getinfo($_curl);
        $curl_error = curl_error($_curl);

        if (!empty($curl_error))
        {
            create_log_error($curl_error);
            throw new Exception($curl_error);
        }

        $http_code = ($curl_info['http_code']) ? $curl_info['http_code'] : 0;
        $error_message = false;
        
        if (400 <= $http_code && $http_code < 500)
        {
        	$error_message = 'Client Error : HTTP ' .$http_code;
        }
        elseif (500 <= $http_code && $http_code < 600)
        {
        	$error_message = 'Server Error : HTTP ' .$http_code;
        }
        
        if ($error_message)
        {
	        if (function_exists("create_log"))
	        {
	            create_log_error($error_message);
	            $info = curl_getinfo($_curl);
	            create_log('', '', '', "Response receive", print_r($info, true));
	        }
            throw new Exception($error_message);
        }
    }
}
?>
